local mod = UpgradeList:NewModule("UI")
local L = LibStub("AceLocale-3.0"):GetLocale("UpgradeList")

local UPGRADELIST_VERSION = GetAddOnMetadata('UpgradeList', 'Version')

local Reputations = {}

function UpgradeList:PassRepsToUI(RepNames)
	Reputations = RepNames
end

local TabNames = {
	{1, "Head", L["Head"]},
	{2, "Neck", L["Neck"]},
	{3, "Shoulder", L["Shoulders"]},
	{4, "Back", L["Back"]},
	{5, "Chest", L["Chest"]},
	{6, "Wrist", L["Wrists"]},
	{7, "Hand", L["Hands"]},
	{8, "Waist", L["Waist"]},
	{9, "Leg", L["Legs"]},
	{10, "Foot", L["Feet"]},
	{11, "Ring", L["Rings"]},
	{12, "Trinket", L["Trinkets"]},
	{13, "Main", L["Main Hand"]},
	{14, "Off", L["Off Hand"]},
	{15, "Ranged", L["Ranged"]},
	{16, "Other", L["Other"]},
}

function UpgradeList:DisableInput()
	UpgradeListMainFrame:EnableMouse(false)
	CloseButton:EnableMouse(false)
	ToggleButton:EnableMouse(false)
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(false)
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(false)
	OptionsFrame:EnableMouse(false)
	ItemEntryBox:EnableMouse(false)
	ULLocationDropDown_Button:EnableMouse(false)
	NoteEntryBox:EnableMouse(false)
	PrimarySpecRadioButton:EnableMouse(false)
	SecondarySpecRadioButton:EnableMouse(false)
	priorityfield:EnableMouse(false)
	AddButton:EnableMouse(false)
	tofield:EnableMouse(false)
	fromfield:EnableMouse(false)
	MoveButton:EnableMouse(false)
	DeleteButton:EnableMouse(false)
	deletefield:EnableMouse(false)
	multifromfield:EnableMouse(false)
	multitofield:EnableMouse(false)
	MultiDeleteButton:EnableMouse(false)
end

function UpgradeList:EnableInput()
	UpgradeListMainFrame:EnableMouse(true)
	CloseButton:EnableMouse(true)
	ToggleButton:EnableMouse(true)
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:EnableMouse(true)
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck:EnableMouse(true)
	OptionsFrame:EnableMouse(true)
	ItemEntryBox:EnableMouse(true)
	ULLocationDropDown_Button:EnableMouse(true)
	NoteEntryBox:EnableMouse(true)
	PrimarySpecRadioButton:EnableMouse(true)
	SecondarySpecRadioButton:EnableMouse(true)
	priorityfield:EnableMouse(true)
	AddButton:EnableMouse(true)
	tofield:EnableMouse(true)
	fromfield:EnableMouse(true)
	MoveButton:EnableMouse(true)
	DeleteButton:EnableMouse(true)
	deletefield:EnableMouse(true)
	multifromfield:EnableMouse(true)
	multitofield:EnableMouse(true)
	MultiDeleteButton:EnableMouse(true)
end

function UpgradeList:StatusBarUpdate(message, color)
	if color == "red" then
		statusbarfont:SetTextColor(1, 0, 0);
	elseif color == "green" then
		statusbarfont:SetTextColor(0, 1, 0);
	else
		statusbarfont:SetTextColor(1, 1, 1);
	end
	statusbarfont:SetText(message);
end

function UpgradeList:ChatFrameMessage(message)
	DEFAULT_CHAT_FRAME:AddMessage("|cFFE08800Upgrade List:|r " .. message)
end

function UpgradeList:ScrollingWindowMessage(message)
	UpgradeListDBVar.profile.messages.scrollwindow:AddMessage(message)
end

function UpgradeList:DrawUI()
	UpgradeList:LoadDropdownItems()
	--Create Scrolling Window
	UpgradeListDBVar.profile.messages.scrollwindow = CreateFrame("ScrollingMessageFrame", "Scrolling Window", UIParent, "ChatFrameTemplate");
	UpgradeListDBVar.profile.messages.scrollwindow:SetPoint("CENTER", 0, 200);
	--[[
	UpgradeListDBVar.profile.messages.scrollwindow:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }});
	--]]
	UpgradeListDBVar.profile.messages.scrollwindow:SetWidth(UIParent:GetWidth() - 200);
	UpgradeListDBVar.profile.messages.scrollwindow:SetHeight(50);
	UpgradeListDBVar.profile.messages.scrollwindow:SetFadeDuration(2);
	UpgradeListDBVar.profile.messages.scrollwindow:SetFading(true);
	UpgradeListDBVar.profile.messages.scrollwindow:SetHyperlinksEnabled(true);
	UpgradeListDBVar.profile.messages.scrollwindow:SetInsertMode("top");
	UpgradeListDBVar.profile.messages.scrollwindow:SetMaxLines(5);
	UpgradeListDBVar.profile.messages.scrollwindow:SetTimeVisible(10);
	UpgradeListDBVar.profile.messages.scrollwindow:SetJustifyH("CENTER");
	UpgradeListDBVar.profile.messages.scrollwindow:SetJustifyV("CENTER");
	UpgradeListDBVar.profile.messages.scrollwindow:SetFontObject(GameFontNormalHuge);
	UpgradeListDBVar.profile.messages.scrollwindow:SetTextColor(1, 1, 1); --white
	UpgradeListDBVar.profile.messages.scrollwindow:EnableMouse(false);
	UpgradeListDBVar.profile.messages.scrollwindow:Show();
	

	--Create Main Frame (UpgradeListMainFrame)
	local UpgradeListMainFrame = CreateFrame("Frame","UpgradeListMainFrame",UIParent);
	UpgradeListMainFrame:SetFrameStrata("DIALOG");
	UpgradeListMainFrame:SetWidth(1257);
	UpgradeListMainFrame:SetHeight(280);
	UpgradeListMainFrame:SetScale(UpgradeListDBVar.profile.mainframe.scale);
	
	UpgradeListMainFrame:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }});

	UpgradeListMainFrame:SetBackdropColor(0,0,0,1);

	UpgradeListMainFrame:SetPoint("TOP",0,-80);
	
	--Makes the Main Frame (and therefore all frames attached to it) draggable
	if UpgradeListDBVar.profile.mainframe.lock == false then
		UpgradeListMainFrame:SetMovable(true)
		UpgradeListMainFrame:EnableMouse(true)
	end
	UpgradeListMainFrame:RegisterForDrag("LeftButton")
	UpgradeListMainFrame:SetScript("OnDragStart", UpgradeListMainFrame.StartMoving)
	UpgradeListMainFrame:SetScript("OnDragStop", UpgradeListMainFrame.StopMovingOrSizing)
	
	tinsert(UISpecialFrames,"UpgradeListMainFrame");


	--Make a title section for the Main Frame 
	local MF_titleBG = UpgradeListMainFrame:CreateTexture(nil,"ARTWORK");
	MF_titleBG:SetTexture("Interface/DialogFrame/UI-DialogBox-Header");
	MF_titleBG:SetWidth(280);
	MF_titleBG:SetHeight(64);
	MF_titleBG:SetPoint("TOP", UpgradeListMainFrame, 0, 12);
	UpgradeListMainFrame.texture = MF_titleBG;

	local MF_titleText = UpgradeListMainFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	MF_titleText:SetText('Upgrade List');
	MF_titleText:SetPoint("TOP", UpgradeListMainFrame, 0, -3);
	
	--Version Identification
	VersionHeading = CreateFrame('Frame', "VersionHeading", UpgradeListMainFrame);
	VersionHeadingFont = VersionHeading:CreateFontString("VersionHeading", "ARTWORK", "GameFontWhiteTiny");
	VersionHeadingFont:SetText("v " .. UPGRADELIST_VERSION);
	VersionHeadingFont:SetPoint("TOPLEFT", UpgradeListMainFrame, "TOPLEFT", 13, -11);
	VersionHeadingFont:SetWidth(VersionHeadingFont:GetStringWidth() + 5);
	VersionHeadingFont:SetHeight(VersionHeadingFont:GetStringHeight());
	
	--Make an invisible frame within the Main Frame that will contain the UpgradeListDBVar.char.SavedOptions.tableUL
	local TableContainer = CreateFrame("Frame", "TableContainer", UpgradeListMainFrame);
	TableContainer:SetWidth(1265);
	TableContainer:SetHeight(235);
	TableContainer:SetPoint("TOPLEFT", UpgradeListMainFrame, "TOPLEFT", 0, -30);
	TableContainer:SetPoint("BOTTOMRIGHT", UpgradeListMainFrame, "BOTTOMRIGHT", 0, -12);

	
	--Create the UpgradeListDBVar.char.SavedOptions.tableUL by defining the columns, rows, height of the rows, then creating the UpgradeListDBVar.char.SavedOptions.tableUL
	local cols = {
		{  -- col 1
			name	= L["Priority"],
			width	= 42,
			defaultsort = "asc",
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 2
			name	= L["iLvl"],
			width	= 35,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 3
			name	= L["Item"],
			width	= 240,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 4
			name	= L["Difficulty"],
			width	= 80,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 5
			name	= L["Instance/Reputation/Point Type/Token"],
			width	= 250,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 6
			name	= L["Boss/Reputation Level/Points Needed/Token Location"],
			width	= 280,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
		{  -- col 7
			name	= L["Note"],
			width	= 280,
			align = "CENTER",
			bgcolor = {
				["r"] = 0.0, 
				["g"] = 0.0, 
				["b"] = 0.0, 
				["a"] = 1.0, 
			},
		},
	};

	local rows = 14
	local height = 15
	local UpgradeListScrollingTable = LibStub("ScrollingTable");
	UpgradeListDBVar.char.SavedOptions.tableUL = UpgradeListScrollingTable:CreateST(cols, rows, height, nil, TableContainer); --UpgradeListDBVar.char.SavedOptions.tableUL
	UpgradeListDBVar.char.SavedOptions.tableUL.frame:SetPoint("CENTER", TableContainer, "CENTER", 0, 0);
	UpgradeListDBVar.char.SavedOptions.tableUL:RegisterEvents({
		["OnClick"] = function (rowFrame, cellFrame, data, cols, row, realrow, column, UpgradeListScrollingTable, ...)
			ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
			if row ~= nil then --if the user isn't clicking in the column header area
				if IsModifiedClick("CTRL") then
					EditNote:Show();
					UpgradeList:DisableInput()
				end
				local ShownSpec
				local ShownSpecLong
				if UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetChecked() then
					ShownSpec = L["Primary"]
					ShownSpecLong = "PrimarySpecList"
				else
					ShownSpec = L["Secondary"]
					ShownSpecLong = "SecondarySpecList"
				end
				for i = 1, #TabNames, 1 do
					if PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == TabNames[i][1] then
						if IsModifiedClick("CTRL") then
							EditNoteOK:SetScript("OnClick", function (self, button, down)
								local newnote = EditNoteBox:GetText();
								EditNote:Hide();
								EditNoteBox:SetText("");
								UpgradeListDBVar.char[ShownSpecLong][TabNames[i][2]].Row[realrow][7] = newnote;
								UpgradeList:EnableInput()
								UpgradeList:RefreshWindow()
							end);
						elseif IsModifiedClick("SHIFT") then
							ChatEdit_InsertLink(UpgradeListDBVar.char[ShownSpecLong][TabNames[i][2]].Row[realrow][3])
						else
							ItemRefTooltip:SetHyperlink(UpgradeListDBVar.char[ShownSpecLong][TabNames[i][2]].Row[realrow][3]);
						end
					end
				end
			end
			ShowUIPanel(ItemRefTooltip)
		end,
	});
	
	--"Close" button
	local CloseButton = CreateFrame('Button', "CloseButton", UpgradeListMainFrame, "UIPanelCloseButton");
	CloseButton:SetPoint("TOPRIGHT", -1, -2);
	CloseButton:SetWidth(40);
	CloseButton:SetHeight(40);
	
	--"ToggleOptions" button
	local ToggleButton = CreateFrame('Button', "ToggleButton", UpgradeListMainFrame, "UIPanelButtonTemplate2");
	ToggleButton:SetPoint("RIGHT", CloseButton, "LEFT", 8, 0);
	ToggleButton:SetWidth(170);
	ToggleButton:SetHeight(20);
	ToggleButton:SetText(L["Toggle Options Window"]);
	ToggleButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:ToggleOptions();
	end);
	
	--"PrimarySpecView" button
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck = CreateFrame('CheckButton', "UpgradeListDBVar.char.SavedOptions.PrimaryCheck", UpgradeListMainFrame, "UICheckButtonTemplate");
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetPoint("RIGHT", ToggleButton, "LEFT", -275, -2);
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetChecked(true);
	_G[UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetName() .. "Text"]:SetText(L["View Primary Spec"]);
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetScript("OnClick", function (self, button, down)
		if GetNumTalentGroups("player") == 1 then
			UpgradeListDBVar.char.SavedOptions.SecondaryCheck:Disable();
			UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetChecked(true);
		else
			UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetChecked(false);
			UpgradeList:RefreshWindow()
		end
	UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetChecked(true);
	end);
	
	--"SecondarySpecView" button
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck = CreateFrame('CheckButton', "UpgradeListDBVar.char.SavedOptions.SecondaryCheck", UpgradeListMainFrame, "UICheckButtonTemplate");
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetPoint("RIGHT", ToggleButton, "LEFT", -125, -2);
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetChecked(false);
	_G[UpgradeListDBVar.char.SavedOptions.SecondaryCheck:GetName() .. "Text"]:SetText(L["View Secondary Spec"]);
	if GetNumTalentGroups("player") == 1 then
		_G[UpgradeListDBVar.char.SavedOptions.SecondaryCheck:GetName() .. "Text"]:SetTextColor(0.5, 0.5, 0.5);
		UpgradeListDBVar.char.SavedOptions.SecondaryCheck:Disable();
	end
	UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetScript("OnClick", function (self, button, down)
		UpgradeListDBVar.char.SavedOptions.SecondaryCheck:SetChecked(true);
		UpgradeListDBVar.char.SavedOptions.PrimaryCheck:SetChecked(false);
		UpgradeList:RefreshWindow()
	end);
	
	--Explanation of what the various clicks on the table do
	ExplanationHeading = CreateFrame('Frame', "ExplanationHeading", UpgradeListMainFrame);
	ExplanationHeadingFont = ExplanationHeading:CreateFontString("ExplanationHeading", "ARTWORK", "GameFontWhiteTiny");
	ExplanationHeadingFont:SetText(L["Clicking a row will show the tooltip for the item, Shift-clicking a row will allow you to add the item link into your chat window, Ctrl-clicking a row will allow you to edit the note for that row."]);
	ExplanationHeadingFont:SetPoint("TOPLEFT", UpgradeListMainFrame, "TOPLEFT", 50, -11);
	ExplanationHeadingFont:SetWidth(500);
	ExplanationHeadingFont:SetHeight(ExplanationHeadingFont:GetStringHeight());
	ExplanationHeadingFont:SetJustifyH("LEFT");

	--Create all the 16 tabs for the equipment slots
	PanelTemplates_SetNumTabs(UpgradeListMainFrame, 16);
	Tab_1 = CreateFrame('Button', "$parentTab1", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_1:SetID(1);
	Tab_1:SetText(L['Head']);
	Tab_1:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -575, 6);
	PanelTemplates_TabResize(Tab_1, 0, 93, 94);
	Tab_1:RegisterForClicks("AnyUp");
	Tab_1:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_1, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);

	Tab_2 = CreateFrame('Button', "$parentTab2", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_2:SetID(2);
	Tab_2:SetText(L['Neck']);
	Tab_2:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -499, 6);
	PanelTemplates_TabResize(Tab_2, 0, 93, 94);
	Tab_2:RegisterForClicks("AnyUp");
	Tab_2:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_2, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_3 = CreateFrame('Button', "$parentTab3", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_3:SetID(3);
	Tab_3:SetText(L['Shoulders']);
	Tab_3:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -423, 6);
	PanelTemplates_TabResize(Tab_3, 0, 93, 94);
	Tab_3:RegisterForClicks("AnyUp");
	Tab_3:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_3, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_4 = CreateFrame('Button', "$parentTab4", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_4:SetID(4);
	Tab_4:SetText(L['Back']);
	Tab_4:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -347, 6);
	PanelTemplates_TabResize(Tab_4, 0, 93, 94);
	Tab_4:RegisterForClicks("AnyUp");
	Tab_4:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_4, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_5 = CreateFrame('Button', "$parentTab5", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_5:SetID(5);
	Tab_5:SetText(L['Chest']);
	Tab_5:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -271, 6);
	PanelTemplates_TabResize(Tab_5, 0, 93, 94);
	Tab_5:RegisterForClicks("AnyUp");
	Tab_5:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_5, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_6 = CreateFrame('Button', "$parentTab6", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_6:SetID(6);
	Tab_6:SetText(L['Wrists']);
	Tab_6:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -195, 6);
	PanelTemplates_TabResize(Tab_6, 0, 93, 94);
	Tab_6:RegisterForClicks("AnyUp");
	Tab_6:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_6, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_7 = CreateFrame('Button', "$parentTab7", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_7:SetID(7);
	Tab_7:SetText(L['Hands']);
	Tab_7:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -119, 6);
	PanelTemplates_TabResize(Tab_7, 0, 93, 94);
	Tab_7:RegisterForClicks("AnyUp");
	Tab_7:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_7, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_8 = CreateFrame('Button', "$parentTab8", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_8:SetID(8);
	Tab_8:SetText(L['Waist']);
	Tab_8:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", -43, 6);
	PanelTemplates_TabResize(Tab_8, 0, 93, 94);
	Tab_8:RegisterForClicks("AnyUp");
	Tab_8:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_8, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_9 = CreateFrame('Button', "$parentTab9", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_9:SetID(9);
	Tab_9:SetText(L['Legs']);
	Tab_9:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 33, 6);
	PanelTemplates_TabResize(Tab_9, 0, 93, 94);
	Tab_9:RegisterForClicks("AnyUp");
	Tab_9:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_9, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_10 = CreateFrame('Button', "$parentTab10", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_10:SetID(10);
	Tab_10:SetText(L['Feet']);
	Tab_10:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 109, 6);
	PanelTemplates_TabResize(Tab_10, 0, 93, 94);
	Tab_10:RegisterForClicks("AnyUp");
	Tab_10:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_10, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_11 = CreateFrame('Button', "$parentTab11", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_11:SetID(11);
	Tab_11:SetText(L['Rings']);
	Tab_11:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 185, 6);
	PanelTemplates_TabResize(Tab_11, 0, 93, 94);
	Tab_11:RegisterForClicks("AnyUp");
	Tab_11:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_11, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_12 = CreateFrame('Button', "$parentTab12", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_12:SetID(12);
	Tab_12:SetText(L['Trinkets']);
	Tab_12:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 261, 6);
	PanelTemplates_TabResize(Tab_12, 0, 93, 94);
	Tab_12:RegisterForClicks("AnyUp");
	Tab_12:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_12, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_13 = CreateFrame('Button', "$parentTab13", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_13:SetID(13);
	Tab_13:SetText(L['Main Hand']);
	Tab_13:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 342, 6);
	PanelTemplates_TabResize(Tab_13, 0, 102, 90);
	Tab_13:RegisterForClicks("AnyUp");
	Tab_13:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_13, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_14 = CreateFrame('Button', "$parentTab14", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_14:SetID(14);
	Tab_14:SetText(L['Off Hand']);
	Tab_14:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 423, 6);
	PanelTemplates_TabResize(Tab_14, 0, 94, 94);
	Tab_14:RegisterForClicks("AnyUp");
	Tab_14:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_14, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_15 = CreateFrame('Button', "$parentTab15", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_15:SetID(15);
	Tab_15:SetText(L['Ranged']);
	Tab_15:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 499, 6);
	PanelTemplates_TabResize(Tab_15, 0, 94, 94);
	Tab_15:RegisterForClicks("AnyUp");
	Tab_15:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_15, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	Tab_16 = CreateFrame('Button', "$parentTab16", UpgradeListMainFrame, "CharacterFrameTabButtonTemplate");
	Tab_16:SetID(16);
	Tab_16:SetText(L['Other']);
	Tab_16:SetPoint("TOP", UpgradeListMainFrame, "BOTTOM", 575, 6);
	PanelTemplates_TabResize(Tab_16, 0, 94, 94);
	Tab_16:RegisterForClicks("AnyUp");
	Tab_16:SetScript("OnClick", function (self, button, down)
		self:SetID(self:GetID());
		PanelTemplates_Tab_OnClick(Tab_16, UpgradeListMainFrame);
		UpgradeList:RefreshWindow()
	end);
	
	PanelTemplates_SetTab(UpgradeListMainFrame, 1);
	UpgradeList:RefreshWindow()
	
	--Create the options frame below the main frame
	local OptionsFrame = CreateFrame("Frame", "OptionsFrame", UpgradeListMainFrame);
	OptionsFrame:SetWidth(850);
	OptionsFrame:SetHeight(330);
	OptionsFrame:SetPoint("TOP",UpgradeListMainFrame,"BOTTOM", 0, -30);
	OptionsFrame:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 12, top = 12, bottom = 11 }});
	
	--Creation of the Item Entry Box
	local ItemEntryBox = CreateFrame('EditBox', "ItemEntryBox", OptionsFrame, "InputBoxTemplate");
	ItemEntryBox:SetPoint("CENTER", 0, 125);
	ItemEntryBox:SetAutoFocus(false);
	ItemEntryBox:SetWidth(130);
    ItemEntryBox:SetHeight(24);
	
	local ItemEntryBoxHeading = CreateFrame('Frame', "ItemEntryBoxHeading", OptionsFrame);
	local HeadingFont = ItemEntryBoxHeading:CreateFontString("ItemHeading", "ARTWORK", "GameFontNormalSmallLeft");
	HeadingFont:SetText(L["Enter your item with an item link or an item ID number:"]);
	HeadingFont:SetPoint("BOTTOM", ItemEntryBox, "TOP");
	HeadingFont:SetWidth(HeadingFont:GetStringWidth() + 5);
	HeadingFont:SetHeight(HeadingFont:GetStringHeight());	

	--Creation of the dropdown menus
	CreateFrame("Button", "ULLocationDropDown_Button", OptionsFrame, "UIDropDownMenuTemplate")
	ULLocationDropDown_Button:ClearAllPoints()
	ULLocationDropDown_Button:SetPoint("TOP", ItemEntryBox, "BOTTOM", 0, -20)
	ULLocationDropDown_Button:Show()
	

	function DropDown_InitializeDropDown(self, level)

		--Top Level
		if level == 1 then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:TopLevel(info, level)

		--Raid Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["Raid"]) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:RaidSubmenu(info, level)
			
		--Dungeon Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["Dungeon"]) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:DungeonSubmenu(info, level)
			
		--Crafted Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["Crafted"]) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:CraftedSubmenu(info, level)
			
		--Quest Reward Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["Quest Reward"]) then
			local info = UIDropDownMenu_CreateInfo()
			--local DropLocation = "Quest Reward"
			
			info.text = L["Quest Reward"]
			info.value = L["Quest Reward"]
			info.notCheckable = true
			function info.func()
				UIDropDownMenu_SetText(ULLocationDropDown_Button, L["Quest Reward"]);
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = L["Quest Reward"];
				CloseDropDownMenus(1);
			end
			UIDropDownMenu_AddButton(info, level)

		--Reputation Reward Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["Reputation Reward"]) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Reputations(info, level)
		
		--Purchasable Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["Purchasable"]) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Purchasable(info, level)
			
		--World Drop Submenu
		elseif (level == 2) and (UIDROPDOWNMENU_MENU_VALUE == L["World Drop"]) then
			local info = UIDropDownMenu_CreateInfo()
			
			info.text = L["World Drop"]
			info.value = L["World Drop"]
			info.notCheckable = true
			function info.func()
				UIDropDownMenu_SetText(ULLocationDropDown_Button, L["World Drop"]);
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = L["World Drop"];
				CloseDropDownMenus(1);
			end
			UIDropDownMenu_AddButton(info, level)

		--List of heroic raids
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == L["Heroic"] .. "Raid") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:HeroicRaids(info, level)
			
		--List of normal raids
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == L["Normal"] .. "Raid") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:NormalRaids(info, level)
		
		--Heroic Dungeons
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == L["Heroic"] .. "Dungeon") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:HeroicDungeons(info, level)
			
		--Normal Dungeons
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == L["Normal"] .. "Dungeon") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:NormalDungeons(info, level)
		
		--Reputation Levels
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == Reputations[1]
			or UIDROPDOWNMENU_MENU_VALUE == Reputations[2]
			or UIDROPDOWNMENU_MENU_VALUE == Reputations[3]
			or UIDROPDOWNMENU_MENU_VALUE == Reputations[4]
			or UIDROPDOWNMENU_MENU_VALUE == Reputations[5]
			or UIDROPDOWNMENU_MENU_VALUE == Reputations[6]
			or UIDROPDOWNMENU_MENU_VALUE == Reputations[7]
			or UIDROPDOWNMENU_MENU_VALUE == GetGuildInfo("player")) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:RepLevels(info, level, UIDROPDOWNMENU_MENU_VALUE)
		
		--Tier Selection for Tokens
		elseif (level == 3) and (UIDROPDOWNMENU_MENU_VALUE == L["Gear Token"]) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:TierSelection(info, level)
			
		--Tier Tokens
		elseif (level == 4) and ((UIDROPDOWNMENU_MENU_VALUE == L["Tier 11"])
		or (UIDROPDOWNMENU_MENU_VALUE == L["Tier 12"])) then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:GearTokens(info, level, UIDROPDOWNMENU_MENU_VALUE)
			
		--Normal Firelands Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 5 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Firelands(info, level, L["Normal"])
			
		--Heroic Firelands Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 5 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Firelands(info, level, L["Heroic"])
			
		--Normal Bastion of Twilight Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 3 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BastionOfTwilight(info, level, L["Normal"])
		
		--Heroic Bastion of Twilight Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 3 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BastionOfTwilight(info, level, L["Heroic"])
			
		--Normal Blackwing Descent Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 4 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BlackwingDescent(info, level, L["Normal"])
			
		--Heroic Blackwing Descent Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 4 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BlackwingDescent(info, level, L["Heroic"])
		
		--Normal Throne of the Four Winds Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 2 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ThroneOfTheFourWinds(info, level, L["Normal"])
			
		--Heroic Throne of the Four Winds Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 2 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ThroneOfTheFourWinds(info, level, L["Heroic"])
			
		--Heroic Baradin Hold Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 1 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BaradinHold(info, level, L["Heroic"])
			
		--Normal Baradin Hold Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "1 1 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BaradinHold(info, level, L["Normal"])
			
		--Heroic Zul'Aman Bosses
		elseif (level==4) and (UIDROPDOWNMENU_MENU_VALUE == "2 4 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ZulAman(info, level)
			
		--Heroic Zul'Gurub Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "2 3 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ZulGurub(info, level)
		
		--Heroic Deadmines Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "2 2 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Deadmines(info, level)
			
		--Heroic Shadowfang Keep Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "2 1 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ShadowfangKeep(info, level)
			
		--Heroic Throne of Tides Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 2 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ThroneOfTides(info, level, L["Heroic"])
			
		--Heroic Blackrock Caverns Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 7 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BlackrockCaverns(info, level, L["Heroic"])
			
		--Heroic Stonecore Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 3 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Stonecore(info, level, L["Heroic"])

		--Heroic Vortex Pinnacle Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 1 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:VortexPinnacle(info, level, L["Heroic"])

		--Heroic Lost City of Tol'vir Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 4 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:LostCityOfTolvir(info, level, L["Heroic"])
			
		--Heroic Grim Batol Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 6 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:GrimBatol(info, level, L["Heroic"])
			
		--Heroic Halls of Origination Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "3 5 Heroic") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:HallsOfOrigination(info, level, L["Heroic"])
			
		--Normal Throne of Tides Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 2 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:ThroneOfTides(info, level, L["Normal"])
			
		--Normal Blackrock Caverns Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 7 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:BlackrockCaverns(info, level, L["Normal"])
		
		--Normal Stonecore Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 3 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:Stonecore(info, level, L["Normal"])
		
		--Normal Vortex Pinnacle Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 1 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:VortexPinnacle(info, level, L["Normal"])
		
		--Normal Lost City of Tol'vir Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 4 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:LostCityOfTolvir(info, level, L["Normal"])
		
		--Normal Grim Batol Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 6 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:GrimBatol(info, level, L["Normal"])
		
		--Normal Halls of Origination Bosses
		elseif (level == 4) and (UIDROPDOWNMENU_MENU_VALUE == "4 5 Normal") then
			local info = UIDropDownMenu_CreateInfo()
			UpgradeList:HallsOfOrigination(info, level, L["Normal"])
		
		end
	end

	function DropDown_DropDownOnLoad(self)
		UIDropDownMenu_Initialize(self, DropDown_InitializeDropDown)
	end

	function ULLocationDropDown_ButtonOnClick(self, button, down)
		local name = self:GetName()
		ToggleDropDownMenu(1, nil, DropDown_DropDown, name, 0, 0)
	end
	
	UIDropDownMenu_Initialize(ULLocationDropDown_Button, DropDown_InitializeDropDown)
	UIDropDownMenu_SetWidth(ULLocationDropDown_Button, 175);
	UIDropDownMenu_SetButtonWidth(ULLocationDropDown_Button, 175)
	UIDropDownMenu_SetSelectedID(ULLocationDropDown_Button, 0)
	UIDropDownMenu_JustifyText(ULLocationDropDown_Button, "CENTER")
	UIDropDownMenu_SetText(ULLocationDropDown_Button, "Location:")

	local DropDownHeading = CreateFrame('Frame', "DropDownHeading", OptionsFrame);
	NoteFont = DropDownHeading:CreateFontString("DropDownHeading", "ARTWORK", "GameFontNormalSmallLeft");
	NoteFont:SetText(L["Select Item Location:"]);
	NoteFont:SetPoint("BOTTOM", ULLocationDropDown_Button, "TOP");
	NoteFont:SetWidth(NoteFont:GetStringWidth() + 5);
	NoteFont:SetHeight(NoteFont:GetStringHeight());	
	
	--Creation of the Note Entry Box
	local NoteEntryBox = CreateFrame('EditBox', "NoteEntryBox", OptionsFrame, "InputBoxTemplate");
	NoteEntryBox:SetPoint("TOP", ULLocationDropDown_Button, "BOTTOM", 0, -20);
	NoteEntryBox:SetAutoFocus(false);
	NoteEntryBox:SetWidth(250);
    NoteEntryBox:SetHeight(24);
	NoteEntryBox:SetMaxLetters(28);
	
	local NoteEntryBoxHeading = CreateFrame('Frame', "NoteEntryBoxHeading", OptionsFrame);
	NoteFont = NoteEntryBoxHeading:CreateFontString("NoteHeading", "ARTWORK", "GameFontNormalSmallLeft");
	NoteFont:SetText(L["Enter an optional note:"]);
	NoteFont:SetPoint("BOTTOM", NoteEntryBox, "TOP");
	NoteFont:SetWidth(NoteFont:GetStringWidth() + 5);
	NoteFont:SetHeight(NoteFont:GetStringHeight());	
	
	--Primary Spec Selector Radio Button
	local PrimarySpecRadioButton = CreateFrame('CheckButton', "PrimarySpecRadioButton", OptionsFrame, "UIRadioButtonTemplate");
	PrimarySpecRadioButton:SetHeight(20);
	PrimarySpecRadioButton:SetWidth(20);
	PrimarySpecRadioButton:SetChecked(true);
	PrimarySpecRadioButton:SetPoint("TOP", NoteEntryBox, "BOTTOM", -95, -25);
	_G[PrimarySpecRadioButton:GetName() .. "Text"]:SetText(L["Primary Spec"]);
	PrimarySpecRadioButton:SetScript("OnClick", function (self, button, down)
		if GetNumTalentGroups("player") == 1 then
			SecondarySpecRadioButton:Disable();
			PrimarySpecRadioButton:SetChecked(true);
		else
			SecondarySpecRadioButton:SetChecked(false);
		end
	PrimarySpecRadioButton:SetChecked(true);
	end);
	
	--Secondary Spec Selector Radio Button
	local SecondarySpecRadioButton = CreateFrame('CheckButton', "SecondarySpecRadioButton", OptionsFrame, "UIRadioButtonTemplate");
	SecondarySpecRadioButton:SetHeight(20);
	SecondarySpecRadioButton:SetWidth(20);
	SecondarySpecRadioButton:SetChecked(false);
	SecondarySpecRadioButton:SetPoint("TOP", NoteEntryBox, "BOTTOM", 20, -25);
	_G[SecondarySpecRadioButton:GetName() .. "Text"]:SetText(L["Secondary Spec"]);
	if GetNumTalentGroups("player") == 1 then
		_G[SecondarySpecRadioButton:GetName() .. "Text"]:SetTextColor(0.5, 0.5, 0.5);
		SecondarySpecRadioButton:Disable();
	end
	SecondarySpecRadioButton:SetScript("OnClick", function (self, button, down)
		SecondarySpecRadioButton:SetChecked(true);
		PrimarySpecRadioButton:SetChecked(false);
	end);
	
	local SpecEntryBoxHeading = CreateFrame('Frame', "SpecEntryBoxHeading", OptionsFrame);
	SpecFont = SpecEntryBoxHeading:CreateFontString("SpecHeading", "ARTWORK", "GameFontNormalSmallLeft");
	SpecFont:SetText(L["Select which spec to which you would like to add this item:"]);
	SpecFont:SetPoint("BOTTOM", NoteEntryBox, "TOP", 0, -47);
	SpecFont:SetWidth(SpecFont:GetStringWidth() + 5);
	SpecFont:SetHeight(SpecFont:GetStringHeight());	
	
	--AddItemLocation
	local priorityfield = CreateFrame('EditBox', "priorityfield", OptionsFrame, "InputBoxTemplate");
	priorityfield:SetPoint("TOP", ItemEntryBox, "BOTTOM", 0, -180);
	priorityfield:SetAutoFocus(false);
	priorityfield:SetWidth(25);
	priorityfield:SetHeight(24);
	
	local PriorityBoxHeading = CreateFrame('Frame', "PriorityBoxHeading", OptionsFrame);
	PriorityFont = PriorityBoxHeading:CreateFontString("SpecHeading", "ARTWORK", "GameFontNormalSmallLeft");
	PriorityFont:SetText(L["Enter the priority number for this item OR \nleave it blank to add the item to bottom of the list:"]);
	PriorityFont:SetJustifyH("CENTER");
	PriorityFont:SetPoint("BOTTOM", priorityfield, "TOP", 0, 5);
	PriorityFont:SetWidth(PriorityFont:GetStringWidth() + 5);
	PriorityFont:SetHeight(PriorityFont:GetStringHeight());	
	
	
	--"Add" button
	local AddButton = CreateFrame('Button', "AddButton", OptionsFrame, "UIPanelButtonTemplate");
	AddButton:SetPoint("TOP", priorityfield, "BOTTOM", 0, -5);
	AddButton:SetWidth(100);
    AddButton:SetHeight(24);
	AddButton:SetText(L["Add Item"]);
	AddButton:SetScript("OnEnter", function (self, motion)
		if ItemEntryBox ~= "" then
			UpgradeList:LoadItemIntoCache(ItemEntryBox);
		end
	end);
	AddButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:AddItem(ItemEntryBox, PrimarySpecRadioButton, UpgradeListMainFrame, priorityfield, NoteEntryBox);
	end);
	
	--Moving heading text
	local MovingHeading = CreateFrame('Frame', "MovingHeading", OptionsFrame);
	MovingHeadingFont = MovingHeading:CreateFontString("MovingHeading", "ARTWORK", "GameFontNormalSmallLeft");
	MovingHeadingFont:SetText(L["In the currently selected tab, move item #:"]);
	MovingHeadingFont:SetPoint("BOTTOM", ItemEntryBox, "TOP", -275, 0);
	MovingHeadingFont:SetWidth(MovingHeadingFont:GetStringWidth() + 5);
	MovingHeadingFont:SetHeight(MovingHeadingFont:GetStringHeight());
	
	--"From" field for moving items
	local fromfield = CreateFrame('EditBox', "fromfield", OptionsFrame, "InputBoxTemplate");
	fromfield:SetPoint("TOP", MovingHeadingFont, "BOTTOM", -25, -2);
	fromfield:SetAutoFocus(false);
	fromfield:SetWidth(25);
	fromfield:SetHeight(24);
	
	--"To" field for moving items
	local tofield = CreateFrame('EditBox', "tofield", OptionsFrame, "InputBoxTemplate");
	tofield:SetPoint("TOP", MovingHeadingFont, "BOTTOM", 25, -3);
	tofield:SetAutoFocus(false);
	tofield:SetWidth(25);
	tofield:SetHeight(24);
	
	--"To" between the To and From fields
	local to = CreateFrame('Frame', "To", OptionsFrame);
	toheadingfont = to:CreateFontString("to", "ARTWORK", "GameFontNormalSmallLeft");
	toheadingfont:SetText(L["to"]);
	toheadingfont:SetPoint("TOP", MovingHeadingFont, "BOTTOM", -2, -9);
	
	--"Move Item" button
	local MoveButton = CreateFrame('Button', "MoveButton", OptionsFrame, "UIPanelButtonTemplate");
	MoveButton:SetPoint("TOP", toheadingfont, "BOTTOM", 0, -10);
	MoveButton:SetWidth(100);
    MoveButton:SetHeight(24);
	MoveButton:SetText(L["Move Item"]);
	MoveButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:MoveItem(fromfield, tofield, UpgradeListMainFrame);
	end);
	
	--"Delete item" heading
	DeletingHeading = CreateFrame('Frame', "DeletingHeading", OptionsFrame);
	DeletingHeadingFont = DeletingHeading:CreateFontString("DeletingHeading", "ARTWORK", "GameFontNormalSmallLeft");
	DeletingHeadingFont:SetText(L["In the currently selected tab, delete item #:"]);
	DeletingHeadingFont:SetPoint("BOTTOM", ItemEntryBox, "TOP", -275, -90);
	DeletingHeadingFont:SetWidth(DeletingHeadingFont:GetStringWidth() + 5);
	DeletingHeadingFont:SetHeight(DeletingHeadingFont:GetStringHeight());
	
	--"Delete Item" button
	local DeleteButton = CreateFrame('Button', "DeleteButton", OptionsFrame, "UIPanelButtonTemplate");
	DeleteButton:SetPoint("TOP", DeletingHeadingFont, "BOTTOM", 0, -25);
	DeleteButton:SetWidth(100);
    DeleteButton:SetHeight(24);
	DeleteButton:SetText(L["Delete Item"]);
	DeleteButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:DeleteItem(deletefield, UpgradeListMainFrame);
	end);
	
	--"Delete Item" field
	local deletefield = CreateFrame('EditBox', "deletefield", OptionsFrame, "InputBoxTemplate");
	deletefield:SetPoint("TOP", DeletingHeadingFont, "BOTTOM", 0, 0);
	deletefield:SetAutoFocus(false);
	deletefield:SetWidth(25);
	deletefield:SetHeight(24);
	
	--MultiDelete heading text
	MultiDeleteHeading = CreateFrame('Frame', "MultiDeleteHeading", OptionsFrame);
	MultiDeleteHeadingFont = MultiDeleteHeading:CreateFontString("MultiDeleteHeading", "ARTWORK", "GameFontNormalSmallLeft");
	MultiDeleteHeadingFont:SetText(L["In the currently selected tab, delete item #'s:"]);
	MultiDeleteHeadingFont:SetPoint("BOTTOM", ItemEntryBox, "TOP", -275, -180);
	MultiDeleteHeadingFont:SetWidth(MultiDeleteHeadingFont:GetStringWidth() + 5);
	MultiDeleteHeadingFont:SetHeight(MultiDeleteHeadingFont:GetStringHeight());
	
	--"From" field for MultiDelete items
	local multifromfield = CreateFrame('EditBox', "multifromfield", OptionsFrame, "InputBoxTemplate");
	multifromfield:SetPoint("TOP", MultiDeleteHeadingFont, "BOTTOM", -38, -2);
	multifromfield:SetAutoFocus(false);
	multifromfield:SetWidth(25);
	multifromfield:SetHeight(24);
	
	--"To" field for MultiDelete items
	local multitofield = CreateFrame('EditBox', "multitofield", OptionsFrame, "InputBoxTemplate");
	multitofield:SetPoint("TOP", MultiDeleteHeadingFont, "BOTTOM", 38, -3);
	multitofield:SetAutoFocus(false);
	multitofield:SetWidth(25);
	multitofield:SetHeight(24);
	
	--"Through" between the To and From fields
	local multithrough = CreateFrame('Frame', "multiThrough", OptionsFrame);
	multithroughheadingfont = to:CreateFontString("to", "ARTWORK", "GameFontNormalSmallLeft");
	multithroughheadingfont:SetText(L["through"]);
	multithroughheadingfont:SetPoint("TOP", MultiDeleteHeadingFont, "BOTTOM", -2, -9);
	
	--"Multi-Delete Item" button
	local MultiDeleteButton = CreateFrame('Button', "MultiDeleteButton", OptionsFrame, "UIPanelButtonTemplate");
	MultiDeleteButton:SetPoint("TOP", multithroughheadingfont, "BOTTOM", 0, -10);
	MultiDeleteButton:SetWidth(100);
    MultiDeleteButton:SetHeight(24);
	MultiDeleteButton:SetText(L["Delete Items"]);
	MultiDeleteButton:SetScript("OnClick", function (self, button, down)
		UpgradeList:DeleteSectionOfItems(multifromfield, multitofield, UpgradeListMainFrame);
	end);
	
	--Status Bar
	local PaneBackdrop  = {
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 3, right = 3, top = 5, bottom = 3 }
	}
	
	statusbar = CreateFrame("Frame", "Status Bar", OptionsFrame)
	statusbar:SetPoint("BOTTOMLEFT", OptionsFrame, "BOTTOMLEFT", 15, 15)
	statusbar:SetPoint("BOTTOMRIGHT", OptionsFrame, "BOTTOMRIGHT", -15, 15)
	statusbar:SetHeight(24)
	statusbar:SetBackdrop(PaneBackdrop)
	statusbar:SetBackdropColor(0.1,0.1,0.1)
	statusbar:SetBackdropBorderColor(0.4,0.4,0.4)
	
	statusbarfont = statusbar:CreateFontString("statusbar", "ARTWORK", "GameFontWhite");
	statusbarfont:SetPoint("LEFT", statusbar, "LEFT", 7, 0);
	statusbarfont:SetHeight(22);
	
	--Frame for entering the number of points needed for certain items
	local PointInput = CreateFrame("Frame", "PointInput", UIParent);
	PointInput:SetFrameStrata("FULLSCREEN");
	PointInput:SetWidth(250);
	PointInput:SetHeight(200);
	
	PointInput:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
	edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
	tile = true, tileSize = 32, edgeSize = 32, 
	insets = { left = 11, right = 11, top = 10, bottom = 10 }});

	PointInput:SetBackdropColor(0,0,0,1);
	PointInput:SetPoint("CENTER", UIParent, "CENTER");
	
	PointInput:Hide();
	
	local PointInputHeading = CreateFrame('Frame', "PointInputHeading", PointInput);
	PointInputFont = PointInputHeading:CreateFontString("PointInputHeading", "ARTWORK", "GameFontNormal");
	PointInputFont:SetText(L["Enter the number\n of points needed\n for this item:"]);
	PointInputFont:SetJustifyH("CENTER");
	PointInputFont:SetPoint("TOP", PointInput, "TOP", 0, -15);
	PointInputFont:SetWidth(230);
	PointInputFont:SetHeight(PointInputFont:GetStringHeight());
	
	local PointInputBox = CreateFrame("Editbox", "PointInputBox", PointInput, "InputBoxTemplate");
	PointInputBox:SetPoint("CENTER", PointInput, "CENTER", 0, 0);
	PointInputBox:SetAutoFocus(true);
	PointInputBox:SetWidth(175);
	PointInputBox:SetHeight(27);
	PointInputBox:SetNumeric(1);
	PointInputBox:SetMaxLetters(5);
	
	local PointInputOK = CreateFrame('Button', "PointInputOK", PointInput, "UIPanelButtonTemplate");
	PointInputOK:SetPoint("BOTTOM", PointInput, "BOTTOM", -50, 20);
	PointInputOK:SetWidth(100);
	PointInputOK:SetHeight(24);
	PointInputOK:SetText(L["Ok"]);
	PointInputOK:SetScript("OnClick", function (self, button, down)
		local pointnum = tonumber(PointInputBox:GetText());
		if pointnum == nil then
			statusbarfont:SetTextColor(1, 0, 0);
			statusbarfont:SetText(L["No number entered."]);
		elseif pointnum >= 0 then
			pointnum = math.floor(pointnum);
			UIDropDownMenu_SetText(ULLocationDropDown_Button, UIDropDownMenu_GetSelectedName(ULLocationDropDown_Button));
			PointInput:Hide();
			PointInputBox:SetText("");
			UpgradeListDBVar.char.SavedOptions.SelectedBoss = pointnum;
			UpgradeListDBVar.char.SavedOptions.SelectedInstance = UIDropDownMenu_GetSelectedName(ULLocationDropDown_Button);
			UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = ""
			UpgradeList:EnableInput()
		else
			statusbarfont:SetTextColor(1, 0, 0);
			statusbarfont:SetText(L["Invalid number."]);
		end
	end);
	
	local PointInputCancel = CreateFrame('Button', "PointInputCancel", PointInput, "UIPanelButtonTemplate");
	PointInputCancel:SetPoint("LEFT", PointInputOK, "RIGHT", 0, 0);
	PointInputCancel:SetWidth(100);
	PointInputCancel:SetHeight(24);
	PointInputCancel:SetText(L["Cancel"]);
	PointInputCancel:SetScript("OnClick", function (self, button, down)
		UIDropDownMenu_SetText(ULLocationDropDown_Button, L["Location:"]);
		PointInput:Hide();
		UpgradeList:EnableInput()
		PointInputBox:SetText("");
	end);
	
	--Frame for selecting whether a weapon is main hand or off hand.
	local WeaponInput = CreateFrame("Frame", "WeaponInput", UIParent);
	WeaponInput:SetFrameStrata("FULLSCREEN");
	WeaponInput:SetWidth(250);
	WeaponInput:SetHeight(200);
	
	WeaponInput:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }});

	WeaponInput:SetBackdropColor(0,0,0,1);
	WeaponInput:SetPoint("CENTER", UIParent, "CENTER");
	
	WeaponInput:Hide();
	
	local WeaponInputHeading = CreateFrame('Frame', "WeaponInputHeading", WeaponInput);
	WeaponInputFont = WeaponInputHeading:CreateFontString("WeaponInputHeading", "ARTWORK", "GameFontNormal");
	WeaponInputFont:SetText(L["Select which list to add this item:"]);
	WeaponInputFont:SetJustifyH("CENTER");
	WeaponInputFont:SetPoint("TOP", WeaponInput, "TOP", 0, -15);
	WeaponInputFont:SetWidth(230);
	WeaponInputFont:SetHeight(WeaponInputFont:GetStringHeight());
	
	--Main Hand weapon checkbox
	local MainHandCheck = CreateFrame('CheckButton', "MainHandCheck", WeaponInput, "UICheckButtonTemplate");
	MainHandCheck:SetPoint("CENTER", WeaponInput, "CENTER", -30, 25);
	MainHandCheck:SetChecked(true);
	_G[MainHandCheck:GetName() .. "Text"]:SetText(L["Main Hand"]);
	MainHandCheck:SetScript("OnClick", function (self, button, down)
		OffHandCheck:SetChecked(false);
		MainHandCheck:SetChecked(true);
	end);
	
	--Off Hand weapon checkbox
	local OffHandCheck = CreateFrame('CheckButton', "OffHandCheck", WeaponInput, "UICheckButtonTemplate");
	OffHandCheck:SetPoint("TOP", MainHandCheck, "BOTTOM", 0, 0);
	OffHandCheck:SetChecked(false);
	_G[OffHandCheck:GetName() .. "Text"]:SetText(L["Off Hand"]);
	OffHandCheck:SetScript("OnClick", function (self, button, down)
		OffHandCheck:SetChecked(true);
		MainHandCheck:SetChecked(false);
	end);
	
	local WeaponInputOK = CreateFrame('Button', "WeaponInputOK", WeaponInput, "UIPanelButtonTemplate");
	WeaponInputOK:SetPoint("BOTTOM", WeaponInput, "BOTTOM", -50, 20);
	WeaponInputOK:SetWidth(100);
	WeaponInputOK:SetHeight(24);
	WeaponInputOK:SetText(L["Ok"]);
	WeaponInputOK:SetScript("OnClick", function (self, button, down)
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(ItemEntryBox:GetText());
		local priorityfieldnum = tonumber(priorityfield:GetText());
		if priorityfieldnum ~= nil then
			priorityfieldnum = math.floor(priorityfieldnum);
		end
		if PrimarySpecRadioButton:GetChecked() then --Adding to Primary Spec List
			if MainHandCheck:GetChecked() then --If the user selects the Main Hand checkbox
				if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char.PrimarySpecList.Main.Count then --Priority # entered was blank or bigger than the list of items
					UpgradeListDBVar.char.PrimarySpecList.Main.Row[UpgradeListDBVar.char.PrimarySpecList.Main.Count] = { UpgradeListDBVar.char.PrimarySpecList.Main.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() };
				elseif priorityfieldnum <= UpgradeListDBVar.char.PrimarySpecList.Main.Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
					table.insert(UpgradeListDBVar.char.PrimarySpecList.Main.Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() });
					for i = UpgradeListDBVar.char.PrimarySpecList.Main.Count, priorityfieldnum, -1 do
						UpgradeListDBVar.char.PrimarySpecList.Main.Row[i][1] = i;
					end
				end
				UpgradeListDBVar.char.PrimarySpecList.Main.Count = UpgradeListDBVar.char.PrimarySpecList.Main.Count + 1;
				statusbarfont:SetTextColor(0, 1, 0);
				statusbarfont:SetText(L["Successfully added "] .. itemLink .. L[" to your Primary Spec's Main tab."]);
				if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 13) and (UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetChecked() == 1) then --If you're viewing the tab you're adding to
					UpgradeList:RefreshWindow()
				end
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil;
				ItemEntryBox:SetText("");
				ItemEntryBox:ClearFocus();
				NoteEntryBox:SetText("");
				NoteEntryBox:ClearFocus();
				priorityfield:SetText("");
				priorityfield:ClearFocus();
				UIDropDownMenu_SetText(ULLocationDropDown_Button, "Location:");
			elseif OffHandCheck:GetChecked() then --If the user selects the Off Hand checkbox
				if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char.PrimarySpecList.Off.Count then --Priority # entered was blank or bigger than the list of items
					UpgradeListDBVar.char.PrimarySpecList.Off.Row[UpgradeListDBVar.char.PrimarySpecList.Off.Count] = { UpgradeListDBVar.char.PrimarySpecList.Off.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() };
				elseif priorityfieldnum <= UpgradeListDBVar.char.PrimarySpecList.Off.Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
					table.insert(UpgradeListDBVar.char.PrimarySpecList.Off.Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() });
					for i = UpgradeListDBVar.char.PrimarySpecList.Off.Count, priorityfieldnum, -1 do
						UpgradeListDBVar.char.PrimarySpecList.Off.Row[i][1] = i;
					end
				end
				UpgradeListDBVar.char.PrimarySpecList.Off.Count = UpgradeListDBVar.char.PrimarySpecList.Off.Count + 1;
				statusbarfont:SetTextColor(0, 1, 0);
				statusbarfont:SetText(L["Successfully added "] .. itemLink .. L[" to your Primary Spec's Off tab."]);
				if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 14) and (UpgradeListDBVar.char.SavedOptions.PrimaryCheck:GetChecked() == 1) then --If you're viewing the tab you're adding to
					UpgradeList:RefreshWindow()
				end
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil;
				ItemEntryBox:SetText("");
				ItemEntryBox:ClearFocus();
				NoteEntryBox:SetText("");
				NoteEntryBox:ClearFocus();
				priorityfield:SetText("");
				priorityfield:ClearFocus();
				UIDropDownMenu_SetText(ULLocationDropDown_Button, L["Location:"]);
			else
				--Can't tell if the user selected main hand or off hand checkbox
			end
		else --Adding to Secondary Spec List
			if MainHandCheck:GetChecked() then --If the user selects the Main Hand checkbox
				if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char.SecondarySpecList.Main.Count then --Priority # entered was blank or bigger than the list of items
					UpgradeListDBVar.char.SecondarySpecList.Main.Row[UpgradeListDBVar.char.SecondarySpecList.Main.Count] = { UpgradeListDBVar.char.SecondarySpecList.Main.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() };
				elseif priorityfieldnum <= UpgradeListDBVar.char.SecondarySpecList.Main.Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
					table.insert(UpgradeListDBVar.char.SecondarySpecList.Main.Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() });
					for i = UpgradeListDBVar.char.SecondarySpecList.Main.Count, priorityfieldnum, -1 do
						UpgradeListDBVar.char.SecondarySpecList.Main.Row[i][1] = i;
					end
				end
				UpgradeListDBVar.char.SecondarySpecList.Main.Count = UpgradeListDBVar.char.SecondarySpecList.Main.Count + 1;
				statusbarfont:SetTextColor(0, 1, 0);
				statusbarfont:SetText(L["Successfully added "] .. itemLink .. L[" to your Secondary Spec's Main tab."]);
				if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 13) and (UpgradeListDBVar.char.SavedOptions.SecondaryCheck:GetChecked() == 1) then --If you're viewing the tab you're adding to
					UpgradeList:RefreshWindow()
				end
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil;
				ItemEntryBox:SetText("");
				ItemEntryBox:ClearFocus();
				NoteEntryBox:SetText("");
				NoteEntryBox:ClearFocus();
				priorityfield:SetText("");
				priorityfield:ClearFocus();
				UIDropDownMenu_SetText(ULLocationDropDown_Button, "Location:");
			elseif OffHandCheck:GetChecked() then --If the user selects the Off Hand checkbox
				if priorityfield:GetText() == "" or priorityfieldnum > UpgradeListDBVar.char.SecondarySpecList.Off.Count then --Priority # entered was blank or bigger than the list of items
					UpgradeListDBVar.char.SecondarySpecList.Off.Row[UpgradeListDBVar.char.SecondarySpecList.Off.Count] = { UpgradeListDBVar.char.SecondarySpecList.Off.Count, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() };
				elseif priorityfieldnum <= UpgradeListDBVar.char.SecondarySpecList.Off.Count and priorityfieldnum > 0 then --If the priority number is equal to or less than the list length and not zero
					table.insert(UpgradeListDBVar.char.SecondarySpecList.Off.Row, priorityfieldnum, { priorityfieldnum, itemLevel, itemLink, UpgradeListDBVar.char.SavedOptions.SelectedDifficulty, UpgradeListDBVar.char.SavedOptions.SelectedInstance, UpgradeListDBVar.char.SavedOptions.SelectedBoss, NoteEntryBox:GetText(), ItemEntryBox:GetText() });
					for i = UpgradeListDBVar.char.SecondarySpecList.Off.Count, priorityfieldnum, -1 do
						UpgradeListDBVar.char.SecondarySpecList.Off.Row[i][1] = i;
					end
				end
				UpgradeListDBVar.char.SecondarySpecList.Off.Count = UpgradeListDBVar.char.SecondarySpecList.Off.Count + 1;
				statusbarfont:SetTextColor(0, 1, 0);
				statusbarfont:SetText(L["Successfully added "] .. itemLink .. L[" to your Secondary Spec's Off tab."]);
				if (PanelTemplates_GetSelectedTab(UpgradeListMainFrame) == 14) and (UpgradeListDBVar.char.SavedOptions.SecondaryCheck:GetChecked() == 1) then --If you're viewing the tab you're adding to
					UpgradeList:RefreshWindow()
				end
				UpgradeListDBVar.char.SavedOptions.SelectedInstance = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedBoss = nil;
				UpgradeListDBVar.char.SavedOptions.SelectedDifficulty = nil;
				ItemEntryBox:SetText("");
				ItemEntryBox:ClearFocus();
				NoteEntryBox:SetText("");
				NoteEntryBox:ClearFocus();
				priorityfield:SetText("");
				priorityfield:ClearFocus();
				UIDropDownMenu_SetText(ULLocationDropDown_Button, L["Location:"]);
			else
				statusbarfont:SetTextColor(0, 1, 0);
				statusbarfont:SetText(L["ERROR: Unable to determine if you selected Main Hand or Off Hand for this weapon"]);
			end
		end
		WeaponInput:Hide();
		OffHandCheck:SetChecked(false);
		MainHandCheck:SetChecked(true);
		UpgradeList:EnableInput();
	end);
	
	local WeaponInputCancel = CreateFrame('Button', "WeaponInputCancel", WeaponInput, "UIPanelButtonTemplate");
	WeaponInputCancel:SetPoint("LEFT", WeaponInputOK, "RIGHT", 0, 0);
	WeaponInputCancel:SetWidth(100);
	WeaponInputCancel:SetHeight(24);
	WeaponInputCancel:SetText(L["Cancel"]);
	WeaponInputCancel:SetScript("OnClick", function (self, button, down)
		WeaponInput:Hide();
		OffHandCheck:SetChecked(false);
		MainHandCheck:SetChecked(true);
		UpgradeList:EnableInput();
	end);
	
	--Frame for asking whether an acquired upgrade is for Main Spec or Off Spec
	local SpecInput = CreateFrame("Frame", "SpecInput", UIParent);
	SpecInput:SetFrameStrata("FULLSCREEN");
	SpecInput:SetWidth(250);
	SpecInput:SetHeight(200);
	
	SpecInput:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }});

	SpecInput:SetBackdropColor(0,0,0,1);
	SpecInput:SetPoint("CENTER", UIParent, "CENTER");
	
	SpecInput:Hide();
	
	local SpecInputHeading = CreateFrame('Frame', "SpecInputHeading", SpecInput);
	SpecInputFont = SpecInputHeading:CreateFontString("SpecInputHeading", "ARTWORK", "GameFontNormal");
	SpecInputFont:SetText(L["Is your newly acquired item for your Main Spec or your Off Spec?"]);
	SpecInputFont:SetJustifyH("CENTER");
	SpecInputFont:SetPoint("TOP", SpecInput, "TOP", 0, -15);
	SpecInputFont:SetWidth(230);
	SpecInputFont:SetHeight(SpecInputFont:GetStringHeight());
	
	--Main Spec selected
	local MainSpecCheck = CreateFrame('CheckButton', "MainSpecCheck", SpecInput, "UICheckButtonTemplate");
	MainSpecCheck:SetPoint("CENTER", SpecInput, "CENTER", -30, 25);
	MainSpecCheck:SetChecked(true);
	_G[MainSpecCheck:GetName() .. "Text"]:SetText(L["Main Spec"]);
	MainSpecCheck:SetScript("OnClick", function (self, button, down)
		OffSpecCheck:SetChecked(false);
		MainSpecCheck:SetChecked(true);
	end);
	
	--Off Spec selected
	local OffSpecCheck = CreateFrame('CheckButton', "OffSpecCheck", SpecInput, "UICheckButtonTemplate");
	OffSpecCheck:SetPoint("TOP", MainHandCheck, "BOTTOM", 0, 0);
	OffSpecCheck:SetChecked(false);
	_G[OffSpecCheck:GetName() .. "Text"]:SetText(L["Off Spec"]);
	OffSpecCheck:SetScript("OnClick", function (self, button, down)
		OffSpecCheck:SetChecked(true);
		MainSpecCheck:SetChecked(false);
	end);
	
	local SpecInputOK = CreateFrame('Button', "SpecInputOK", SpecInput, "UIPanelButtonTemplate");
	SpecInputOK:SetPoint("BOTTOM", SpecInput, "BOTTOM", -50, 20);
	SpecInputOK:SetWidth(100);
	SpecInputOK:SetHeight(24);
	SpecInputOK:SetText(L["Ok"]);
		
	local SpecInputCancel = CreateFrame('Button', "SpecInputCancel", SpecInput, "UIPanelButtonTemplate");
	SpecInputCancel:SetPoint("LEFT", SpecInputOK, "RIGHT", 0, 0);
	SpecInputCancel:SetWidth(100);
	SpecInputCancel:SetHeight(24);
	SpecInputCancel:SetText(L["Cancel"]);
	SpecInputCancel:SetScript("OnClick", function (self, button, down)
		SpecInput:Hide();
		OffSpecCheck:SetChecked(false);
		MainSpecCheck:SetChecked(true);
		UpgradeList:EnableInput();
	end);
	
	--Frame for asking whether an acquired upgrade is for Main Hand or Off Hand
	local MainHandorOffHand = CreateFrame("Frame", "MainHandorOffHand", UIParent);
	MainHandorOffHand:SetFrameStrata("FULLSCREEN");
	MainHandorOffHand:SetWidth(250);
	MainHandorOffHand:SetHeight(200);
	
	MainHandorOffHand:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
		edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
		tile = true, tileSize = 32, edgeSize = 32, 
		insets = { left = 11, right = 11, top = 10, bottom = 10 }});

	MainHandorOffHand:SetBackdropColor(0,0,0,1);
	MainHandorOffHand:SetPoint("CENTER", UIParent, "CENTER");
	
	MainHandorOffHand:Hide();
	
	local MainHandorOffHandHeading = CreateFrame('Frame', "MainHandorOffHandHeading", MainHandorOffHand);
	MainHandorOffHandFont = MainHandorOffHandHeading:CreateFontString("MainHandorOffHandHeading", "ARTWORK", "GameFontNormal");
	MainHandorOffHandFont:SetText(L["Is your newly acquired item for your Main Hand or your Off Hand?"]);
	MainHandorOffHandFont:SetJustifyH("CENTER");
	MainHandorOffHandFont:SetPoint("TOP", MainHandorOffHand, "TOP", 0, -15);
	MainHandorOffHandFont:SetWidth(230);
	MainHandorOffHandFont:SetHeight(MainHandorOffHandFont:GetStringHeight());
	
	--Main Hand weapon check
	local WeaponInMain = CreateFrame('CheckButton', "WeaponInMain", MainHandorOffHand, "UICheckButtonTemplate");
	WeaponInMain:SetPoint("CENTER", MainHandorOffHand, "CENTER", -30, 25);
	WeaponInMain:SetChecked(true);
	_G[WeaponInMain:GetName() .. "Text"]:SetText(L["Main Hand"]);
	WeaponInMain:SetScript("OnClick", function (self, button, down)
		WeaponInOff:SetChecked(false);
		WeaponInMain:SetChecked(true);
	end);
	
	--Off Hand weapon check
	local WeaponInOff = CreateFrame('CheckButton', "WeaponInOff", MainHandorOffHand, "UICheckButtonTemplate");
	WeaponInOff:SetPoint("TOP", MainHandCheck, "BOTTOM", 0, 0);
	WeaponInOff:SetChecked(false);
	_G[WeaponInOff:GetName() .. "Text"]:SetText(L["Off Hand"]);
	WeaponInOff:SetScript("OnClick", function (self, button, down)
		WeaponInOff:SetChecked(true);
		WeaponInMain:SetChecked(false);
	end);
	
	local MainHandorOffHandOK = CreateFrame('Button', "MainHandorOffHandOK", MainHandorOffHand, "UIPanelButtonTemplate");
	MainHandorOffHandOK:SetPoint("BOTTOM", MainHandorOffHand, "BOTTOM", -50, 20);
	MainHandorOffHandOK:SetWidth(100);
	MainHandorOffHandOK:SetHeight(24);
	MainHandorOffHandOK:SetText(L["Ok"]);
	
	local MainHandorOffHandCancel = CreateFrame('Button', "MainHandorOffHandCancel", MainHandorOffHand, "UIPanelButtonTemplate");
	MainHandorOffHandCancel:SetPoint("LEFT", MainHandorOffHandOK, "RIGHT", 0, 0);
	MainHandorOffHandCancel:SetWidth(100);
	MainHandorOffHandCancel:SetHeight(24);
	MainHandorOffHandCancel:SetText(L["Cancel"]);
	MainHandorOffHandCancel:SetScript("OnClick", function (self, button, down)
		MainHandorOffHand:Hide();
		WeaponInOff:SetChecked(false);
		WeaponInMain:SetChecked(true);
		UpgradeList:EnableInput();
	end);
	
	--Frame for entering a new note
	local EditNote = CreateFrame("Frame", "EditNote", UIParent);
	EditNote:SetFrameStrata("FULLSCREEN");
	EditNote:SetWidth(360);
	EditNote:SetHeight(150);
	
	EditNote:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
	edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
	tile = true, tileSize = 32, edgeSize = 32, 
	insets = { left = 11, right = 11, top = 10, bottom = 10 }});

	EditNote:SetBackdropColor(0,0,0,1);
	EditNote:SetPoint("CENTER", UIParent, "CENTER");
	
	EditNote:Hide();
	
	local EditNoteHeading = CreateFrame('Frame', "EditNoteHeading", EditNote);
	EditNoteFont = EditNoteHeading:CreateFontString("EditNoteHeading", "ARTWORK", "GameFontNormal");
	EditNoteFont:SetText(L["Enter your new note:"]);
	EditNoteFont:SetJustifyH("CENTER");
	EditNoteFont:SetPoint("TOP", EditNote, "TOP", 0, -15);
	EditNoteFont:SetWidth(230);
	EditNoteFont:SetHeight(EditNoteFont:GetStringHeight());
	
	local EditNoteBox = CreateFrame("Editbox", "EditNoteBox", EditNote, "InputBoxTemplate");
	EditNoteBox:SetPoint("CENTER", EditNote, "CENTER", 0, 0);
	EditNoteBox:SetAutoFocus(true);
	EditNoteBox:SetWidth(285);
	EditNoteBox:SetHeight(27);
	EditNoteBox:SetMaxLetters(28);
	
	local EditNoteOK = CreateFrame('Button', "EditNoteOK", EditNote, "UIPanelButtonTemplate");
	EditNoteOK:SetPoint("BOTTOM", EditNote, "BOTTOM", -50, 20);
	EditNoteOK:SetWidth(100);
	EditNoteOK:SetHeight(24);
	EditNoteOK:SetText(L["Ok"]);
	
	local EditNoteCancel = CreateFrame('Button', "EditNoteCancel", EditNote, "UIPanelButtonTemplate");
	EditNoteCancel:SetPoint("LEFT", EditNoteOK, "RIGHT", 0, 0);
	EditNoteCancel:SetWidth(100);
	EditNoteCancel:SetHeight(24);
	EditNoteCancel:SetText(L["Cancel"]);
	EditNoteCancel:SetScript("OnClick", function (self, button, down)
		EditNote:Hide();
		UpgradeList:EnableInput();
		EditNoteBox:SetText("");
	end);
	
	UpgradeListMainFrame:Hide();
end